<?php
/**
 * Copyright (c) 2024 iSmartFrame.
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 * @author iSmartFrame
 * @copyright 2024 iSmartFrame
 * @license LICENSE.txt
 */
date_default_timezone_set('Europe/Rome');
if (!defined('_PS_VERSION_')) {
    exit;
}
class IsfPlugin extends Module
{
    public function __construct()
    {
        $this->module_key = '3047f11246fe34998f015202c49db0b8';
        $this->name = 'isfplugin';
        $this->tab = 'administration';
        $this->version = '1.0.0';
        $this->author = 'iSmartFrame';
        $this->need_instance = 0;
        $this->need_instance = 0;
        $this->ps_versions_compliancy = [
            'min' => '1.7.6.0',
            'max' => _PS_VERSION_,
        ];
        $this->bootstrap = true;
        $this->icon = 'logo.png';
        parent::__construct();
        $this->displayName = $this->l('iSF Plugin');
        $this->description = $this->l('Integrates iSmartFrame platform with Prestashop.');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
        $this->logFile = _PS_MODULE_DIR_ . $this->name . '/install.log';
        if (!Configuration::get('CACHETAGHEADER_NAME')) {
            $this->warning = $this->l('No name provided');
        }
    }
    public function install()
    {
        $hooksRegistered = parent::install()
            && $this->registerHook('actionDispatcher')
            && $this->registerHook('actionOutputHTMLBefore')
            && $this->registerHook('actionProductUpdate')
            && $this->registerHook('actionCategoryUpdate')
            && $this->registerHook('actionGetAdminToolbarButtons')
            && $this->registerHook('displayAdminProductsExtra')
            && $this->registerHook('displayBackOfficeHeader')
            && $this->registerHook('actionObjectUpdateAfter')
            && $this->registerHook('displayDashboardToolbarTopMenu');
        return $hooksRegistered && $this->installDb();
    }

    public function hookDisplayBackOfficeHeader($params)
    {
        $this->context->controller->addCSS($this->_path . 'views/css/styles.css');
    }

    private function installDb()
    {
        $sql_cache = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'cache_data` (
        `cache_id` VARCHAR(255) NOT NULL,
        `value` TEXT NOT NULL,
        PRIMARY KEY (`cache_id`)
    ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;';
        $sql_logs = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'isfplugin_logs` (
        `id_log` INT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
        `timestamp` DATETIME NOT NULL,
        `operationType` VARCHAR(255) NOT NULL,
        `result` VARCHAR(255) NOT NULL,
        `userId` VARCHAR(255) NULL,
        `object` TEXT NULL,
        INDEX (`timestamp`)
    ) ENGINE=' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8;';
        $result_cache = Db::getInstance()->execute($sql_cache);
        $result_logs = Db::getInstance()->execute($sql_logs);
        return $result_cache && $result_logs;
    }
    public function hookDisplayDashboardToolbarTopMenu(array $params): void
    {
        $current_url = Tools::getCurrentUrlProtocolPrefix() . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
        if (strpos($current_url, 'edit?') !== false) {
            $apiKey = Configuration::get('ISFPLUGIN_API_KEY');
            $apiUrl = 'https://app.ismartframe.com/api/v1/cache/purge?api_key=' . urlencode($apiKey);
            $pattern = '/\/(\d+)\/edit\?/';
            $controller_name = Tools::getValue('controller');
            $output = '';
            if (preg_match($pattern, $current_url, $matches)) {
                if ($controller_name === 'AdminCategories') {
                    $output = $this->context->link->getCategoryLink($matches[1]);
                } elseif ($controller_name === 'AdminCmsContent') {
                    $output = $this->context->link->getCMSLink($matches[1]);
                }
                $parsedUrl = parse_url($output, PHP_URL_PATH);
                if ($output) {
                    $this->context->smarty->assign([
                        'purge_url' => addslashes($parsedUrl),
                        'api_url' => addslashes($apiUrl),
                    ]);
                    $this->context->smarty->fetch(__FILE__, 'views/templates/hook/form.tpl');
                    // $this->context->smarty->display('views/templates/hook/form.tpl');
                }
            }
        }
        static $requestProcessed = false;
        if ($_SERVER['REQUEST_METHOD'] === 'POST' && Tools::getIsset(Tools::getValue('purge_url'), Tools::getValue('api_url')) && !$requestProcessed) {
            $this->purgeCacheServerToServer(Tools::getValue('purge_url'), Tools::getValue('api_url'));
            $requestProcessed = true;
        }
    }
    private function purgeCacheServerToServer(string $purgeUrl, string $apiUrl): void
    {
        $adminInfo = $this->getLoggedAdminInfo();
        if (Configuration::get('ISFPLUGIN_API_KEY_VERIFIED')) {
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $apiUrl);
            curl_setopt($ch, CURLOPT_POST, 1);
            $postData = json_encode(['url' => [$purgeUrl]]);
            $postData = str_replace('\/', '/', $postData);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);
            curl_setopt($ch, CURLOPT_HTTPHEADER, [
                'Content-Type: application/json',
                'Content-Length: ' . strlen($postData),
            ]);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            $response = curl_exec($ch);
            $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);
            $alertData = [
                'html' => '',
                'id' => 'cache-purge-alert',
            ];
            if ($httpCode === 200) {
                $responseData = json_decode($response, true);
                if (json_last_error() === JSON_ERROR_NONE && isset($responseData['result']) && $responseData['result'] === true) {
                    $this->addLog('URL', 'OK', $adminInfo, $postData);
                    $alertData['html'] = 'Cache purged successfully for ' . htmlspecialchars($purgeUrl);
                    $alertData['type'] = 'success';
                } else {
                    $this->addLog('URL', 'KO: API response', $adminInfo, $postData);
                    $alertData['html'] = 'Failed to purge cache. The API response indicated a failure.';
                    $alertData['type'] = 'danger';
                }
            } else {
                $this->addLog('URL', 'KO: ' . $httpCode, $adminInfo, $postData);
                $alertData['html'] = 'Failed to purge cache. HTTP error code: ' . htmlspecialchars($httpCode);
                $alertData['type'] = 'danger';
            }
            if (!$httpCode || $httpCode !== 200) {
                $this->addLog('URL', 'KO: API key not verified', $adminInfo, $purgeUrl);
                $alertData['html'] = 'Failed to purge cache. API key not verified.';
                $alertData['type'] = 'danger';
            }
            $this->context->smarty->assign([
                'alertData' => $alertData,
            ]);
            // $this->context->smarty->display('views/templates/hook/alert.tpl');
            $this->context->smarty->fetch(__FILE__, 'views/templates/hook/alert.tpl');
        }
    }
    public function hookDisplayAdminProductsExtra(array $params): string
    {
        $productId = (int) $params['id_product'];
        if (!$productId) {
            return $this->displayError($this->l('Product not found.'));
        }
        $productUrl = $this->context->link->getProductLink($productId);
        $parsedUrl = parse_url($productUrl, PHP_URL_PATH);
        if (Configuration::get('ISFPLUGIN_API_KEY_VERIFIED')) {
            $apiKey = Configuration::get('ISFPLUGIN_API_KEY');
            $apiUrl = 'https://app.ismartframe.com/api/v1/cache/purge?api_key=' . urlencode($apiKey);
            $current_url = Tools::getCurrentUrlProtocolPrefix() . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
            if (strpos($current_url, 'purgeCache=1') !== false) {
                $this->purgeCacheServerToServerProduct($parsedUrl, $apiUrl);
            }
            $this->context->smarty->assign([
                'l' => $this->l('Purge cache for this page'),
            ]);

            return $this->context->smarty->fetch($this->local_path . 'views/templates/hook/purge_cache.tpl');
        } else {
            $adminInfo = $this->getLoggedAdminInfo();
            $this->addLog('URL', 'KO: API key not verified', $adminInfo, $parsedUrl);
            return $this->displayError($this->l('API key not verified.'));
        }
    }
    private function purgeCacheServerToServerProduct(string $purgeUrl, string $apiUrl): string
    {
        $adminInfo = $this->getLoggedAdminInfo();
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $apiUrl);
        curl_setopt($ch, CURLOPT_POST, 1);
        $postData = json_encode(['url' => [$purgeUrl]]);
        $postData = str_replace('\/', '/', $postData);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Content-Type: application/json',
            'Content-Length: ' . strlen($postData),
        ]);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        $alertType = '';
        $errorMessage = '';
        if ($httpCode === 200) {
            $responseData = json_decode($response, true);
            if (json_last_error() === JSON_ERROR_NONE && isset($responseData['result']) && $responseData['result'] === true) {
                $this->addLog('URL', 'OK', $adminInfo, $postData);
                $alertType = 'success';
                $purgeUrl = htmlspecialchars($purgeUrl);
            } else {
                $this->addLog('URL', 'KO: API response', $adminInfo, $postData);
                $alertType = 'error';
                $errorMessage = 'The API response indicated a failure.';
            }
        } else {
            $this->addLog('URL', 'KO: ' . $httpCode, $adminInfo, $postData);
            $alertType = 'error';
            $errorMessage = 'HTTP error code: ' . htmlspecialchars($httpCode);
        }
        $this->context->smarty->assign([
            'alertType' => $alertType,
            'errorMessage' => $errorMessage,
            'purgeUrl' => $purgeUrl ?? '',
        ]);
        $html = $this->context->smarty->display('views/templates/hook/cache_alert.tpl');
        return $html;
    }
    public function hookActionDispatcher($params)
    {
        $context = Context::getContext();
        $controller = $context->controller;
        $cache_tags = ['home'];
        $id = 'home';
        if ($controller instanceof CategoryController) {
            $categoryId = (int) Tools::getValue('id_category');
            if ($categoryId) {
                $id = 'c' . $categoryId;
                $category = new Category($categoryId, $context->language->id);
                if (Validate::isLoadedObject($category)) {
                    $cache_tags[] = 'c' . $category->id;
                }
            }
        } elseif ($controller instanceof ProductController) {
            $productId = (int) Tools::getValue('id_product');
            if ($productId) {
                $id = 'p' . $productId;
                $product = new Product($productId, false, $context->language->id);
                if (Validate::isLoadedObject($product)) {
                    $cache_tags[] = 'p' . $product->id;
                    $categories = $product->getCategories();
                    foreach ($categories as $categoryId) {
                        $cache_tags[] = 'c' . (int) $categoryId;
                    }
                }
            }
        } elseif ($controller instanceof CmsController) {
            $cmsId = (int) Tools::getValue('id_cms');
            $cmsCategoryId = (int) Tools::getValue('id_cms_category');
            if ($cmsId) {
                $id = 'cms' . $cmsId;
                $cms = new CMS($cmsId, $context->language->id);
                if (Validate::isLoadedObject($cms)) {
                    $cache_tags[] = 'cms' . $cms->id;
                }
            } elseif ($cmsCategoryId) {
                $id = 'cms_c' . $cmsCategoryId;
                $cmsCategory = new CMSCategory($cmsCategoryId, $context->language->id);
                if (Validate::isLoadedObject($cmsCategory)) {
                    $cache_tags[] = 'cms_c' . $cmsCategory->id;
                }
            }
        } elseif ($controller instanceof CmsCategoryController) {
            $cmsCategoryId = (int) Tools::getValue('id_cms_category');
            if ($cmsCategoryId) {
                $id = 'cms_c' . $cmsCategoryId;
                $cmsCategory = new CMSCategory($cmsCategoryId, $context->language->id);
                if (Validate::isLoadedObject($cmsCategory)) {
                    $cache_tags[] = 'cms_c' . $cmsCategory->id;
                }
            }
        } elseif ($controller instanceof PostController) {
            $postId = (int) Tools::getValue('id_post');
            if ($postId) {
                $id = 'post' . $postId;
                $post = new Post($postId, $context->language->id);
                if (Validate::isLoadedObject($post)) {
                    $cache_tags[] = 'post_' . $post->id;
                    $postCategories = $post->getCategories();
                    foreach ($postCategories as $categoryId) {
                        $cache_tags[] = 'c' . (int) $categoryId;
                    }
                }
            }
        } elseif ($controller instanceof PostCategoryController) {
            $postCategoryId = (int) Tools::getValue('id_post_category');
            if ($postCategoryId) {
                $id = 'post_c' . $postCategoryId;
                $postCategory = new PostCategory($postCategoryId, $context->language->id);
                if (Validate::isLoadedObject($postCategory)) {
                    $cache_tags[] = 'post_c' . $postCategory->id;
                }
            }
        } else {
        }
        $this->context->cookie->__set('cache_tags_temp', implode(',', $cache_tags));
        if ($id) {
            $this->updateOrAddItem($id, implode(',', $cache_tags));
        }
    }
    private function updateOrAddItem($cache_id, $value)
    {
        $sql = 'SELECT COUNT(*) FROM `' . _DB_PREFIX_ . 'cache_data` WHERE `cache_id` = \'' . $cache_id . '\'';
        $exists = Db::getInstance()->getValue($sql);

        if ($exists) {
            $sql = 'UPDATE `' . _DB_PREFIX_ . 'cache_data` SET `value` = \'' . pSQL($value) . '\' WHERE `cache_id` = \'' . $cache_id . '\'';
        } else {
            $sql = 'INSERT INTO `' . _DB_PREFIX_ . 'cache_data` (`cache_id`, `value`) VALUES (\'' . $cache_id . '\', \'' . pSQL($value) . '\')';
        }

        return Db::getInstance()->execute($sql);
    }
    public function getItemByCacheId($cache_id)
    {
        $cache_id = pSQL($cache_id);

        $sql = 'SELECT `value` FROM `' . _DB_PREFIX_ . 'cache_data` WHERE `cache_id` = \'' . $cache_id . '\'';

        $result = Db::getInstance()->getValue($sql);

        return $result !== false ? $result : null;
    }
    public function hookActionOutputHTMLBefore($params)
    {
        if ($this->context->controller->controller_type == 'front') {
            $cache_tags = $this->context->cookie->__get('cache_tags_temp');
            if ($cache_tags) {
                $cache_tags_array = array_unique(explode(',', $cache_tags));

                $header_value = implode(',', $cache_tags_array);
                header('cache-tag: ' . $header_value);
            }

            $this->context->cookie->__unset('cache_tags_temp');
        }
    }
    private function isHtmlResponse()
    {
        $acceptHeader = $_SERVER['HTTP_ACCEPT'] ?? '';
        $is_html = strpos($acceptHeader, 'text/html') !== false;
        return $is_html;
    }
    public function hookActionProductUpdate($params)
    {
        $productId = (int) $params['id_product'];
        if ($productId) {
            $productUrl = $this->context->link->getProductLink($productId);
            $this->handleObjectUpdate($params['product'], 'p', $productUrl);
        }
    }
    public function hookActionCategoryUpdate($params)
    {
        $categoryId = (int) $params['category']->id;
        if ($categoryId) {
            $categoryUrl = $this->context->link->getCategoryLink($categoryId);
            $this->handleObjectUpdate($params['category'], 'c', $categoryUrl);
        }
    }
    public function hookActionObjectUpdateAfter($params)
    {
        if (isset($params['object'])) {
            $object = $params['object'];
            $url = $this->context->link->getCMSLink($object->id);
            if ($object instanceof CMS) {
                $this->handleObjectUpdate($object, 'cms', $url);
            } elseif ($object instanceof CMSCategory) {
                $this->handleObjectUpdate($object, 'cms_c', $url);
            }
        }
    }
    private function handleObjectUpdate($object, $prefix = '', $url = null)
    {
        static $processedIds = [];
        $adminInfo = $this->getLoggedAdminInfo();
        if (is_object($object)) {
            if (isset($object->id)) {
                $id = $prefix . $object->id;
                if ([$id, $processedIds]) {
                    return;
                }
                $processedIds[] = $id;
                $cacheTag = $this->getItemByCacheId($id);
                if ($cacheTag) {
                    $this->purgeCacheByTag($cacheTag, $url);
                }
            } else {
                $this->addLog('TAG', 'KO: no ID', $adminInfo, $url);
            }
        } else {
            $this->addLog('TAG', 'KO: Invalid object', $adminInfo, $url);
        }
    }
    private function purgeCacheByTag($cacheTag, $object = null)
    {
        $adminInfo = $this->getLoggedAdminInfo();
        $url = 'https://app.ismartframe.com/api/v1/cache/purge/tag?api_key=' . urlencode(Configuration::get('ISFPLUGIN_API_KEY'));
        $array = explode(',', $cacheTag);
        $data = json_encode(['tag' => $array]);
        if (Configuration::get('ISFPLUGIN_API_KEY_VERIFIED')) {
            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt(
                $ch,
                CURLOPT_HTTPHEADER,
                [
                    'Content-Type: application/json',
                    'Content-Length: ' . strlen($data),
                ]
            );
            $result = curl_exec($ch);
            if (curl_errno($ch)) {
                $error_code = curl_errno($ch);
                $this->addLog('TAG', 'KO: ' . $error_code, $adminInfo, $data);
            } else {
                $response = json_decode($result, true);

                if (isset($response['result']) && $response['result'] === false) {
                    $this->addLog('TAG', 'KO: ' . $response['error'], $adminInfo, $data);
                } elseif (isset($response['result']) && $response['result'] === true) {
                    $this->addLog('TAG', 'OK', $adminInfo, $data);
                } else {
                    $this->addLog('TAG', 'KO: Unknown error', $adminInfo, $data);
                }
            }
            curl_close($ch);
        } else {
            $this->addLog('TAG', 'KO: API key not verified', $adminInfo, $data);
        }
    }
    private function addLog($operationType, $result, $userId = null, $object = null)
    {
        $this->cleanOldLogs();
        $operationType = pSQL($operationType);
        $result = pSQL($result);
        $object = is_null($object) ? 'NULL' : "'" . pSQL($object) . "'";
        $sql = 'INSERT INTO `' . _DB_PREFIX_ . 'isfplugin_logs` (
            `timestamp`,
            `operationType`,
            `result`,
            `userId`,
            `object`
        ) VALUES (
            NOW(),
            "' . $operationType . '",
            "' . $result . '",
            ' . (is_null($userId) ? 'NULL' : (int) $userId) . ',
            ' . $object . '
        )';
        $success = Db::getInstance()->execute($sql);
        if (!$success) {
            error_log('Errore durante l\'inserimento del log: ' . Db::getInstance()->getMsgError());
        }
    }
    private function cleanOldLogs()
    {
        $date_limit = date('Y-m-d H:i:s', strtotime('-$max_age_minutes minutes'));
        $sql = 'DELETE FROM `' . _DB_PREFIX_ . 'isfplugin_logs` WHERE `timestamp` < \'' . pSQL($date_limit) . '\'';
        return Db::getInstance()->execute($sql);
    }
    private function getLogs($limit = 100)
    {
        $sql = 'SELECT `id_log`, `timestamp`, `operationType`, `result`, `userId`, `object`
                FROM `' . _DB_PREFIX_ . 'isfplugin_logs`
                ORDER BY `timestamp` DESC
                LIMIT ' . (int) $limit;
        $results = Db::getInstance()->executeS($sql);
        return $results ? $results : [];
    }
    public function uninstall()
    {
        $hooksUnregistered = $this->unregisterHook('actionDispatcher')
            && $this->unregisterHook('displaybackOfficeHeader')
            && $this->unregisterHook('actionOutputHTMLBefore')
            && $this->unregisterHook('actionProductUpdate')
            && $this->unregisterHook('actionCategoryUpdate')
            && $this->unregisterHook('actionGetAdminToolbarButtons')
            && $this->unregisterHook('displayAdminProductsExtra')
            && $this->unregisterHook('actionObjectUpdateAfter')
            && $this->unregisterHook('displayDashboardToolbarTopMenu');
        return parent::uninstall() && $hooksUnregistered && $this->uninstallDb();
    }
    private function uninstallDb()
    {
        $queries = [
            'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . 'cache_data;',
            'DROP TABLE IF EXISTS ' . _DB_PREFIX_ . 'isfplugin_logs;',
        ];
        $db = Db::getInstance();
        foreach ($queries as $sql) {
            if (!$db->execute($sql)) {
                return false;
            }
        }
        return true;
    }
    public function getContent()
    {
        $output = '';
        $adminInfo = $this->getLoggedAdminInfo();
        if (Tools::isSubmit('verifyApiKey')) {
            $apiKey = Tools::getValue('ISFPLUGIN_API_KEY');
            if (empty($apiKey)) {
                $output .= $this->displayError($this->l('API Key cannot be empty.'));
                $this->addLog('KEY', 'KO: Empty key', $adminInfo, null);
                return $output . $this->renderForm();
            }
            if ($apiKey == Configuration::get('ISFPLUGIN_API_KEY') && Configuration::get('ISFPLUGIN_API_KEY_VERIFIED')) {
                $output .= $this->displayError($this->l('This API Key is already verified.'));
                $this->addLog('KEY', 'KO: Key already verified', $adminInfo, $apiKey);
                return $output . $this->renderForm();
            }
            $response = $this->verifyApiKey($apiKey);
            $res = json_decode($response['response'], true);
            $status = $response['status'];
            if ($status == 200) {
                $domain = $res['domain'];
                $domainKey = Tools::getValue('ISFPLUGIN_DOMAIN');
                if ($domain != $domainKey) {
                    $output .= $this->displayError($this->l('API key verification failed. The domain does not match the API key.'));
                    $this->addLog('KEY', 'KO: Domain mismatch', $adminInfo, $apiKey);
                    return $output . $this->renderForm();
                }
                Configuration::updateValue('ISFPLUGIN_API_KEY_VERIFIED', true);
                Configuration::updateValue('ISFPLUGIN_API_KEY', $apiKey);
                $output .= $this->displayConfirmation($this->l('API Key verified successfully.'));
                return $output . $this->renderForm();
            } else {
                Configuration::updateValue('ISFPLUGIN_API_KEY_VERIFIED', false);
                $output .= $this->displayError($this->l('API key verification failed. We did not save your API key. Please check your API key.'));
                $this->addLog('KEY', 'KO: Verification failed', $adminInfo, $apiKey);
                return $output . $this->renderForm();
            }
        }
        if (Tools::isSubmit('purge_cache_action') && Tools::getValue('purge_cache_action') == 1) {
            $cacheCooldownTime = 300;
            $lastPurgeTimestamp = Configuration::get('ISFPLUGIN_PurgeCache');
            $currentTimestamp = time();
            if ($lastPurgeTimestamp && ($currentTimestamp - $lastPurgeTimestamp < $cacheCooldownTime)) {
                $timeRemaining = $cacheCooldownTime - ($currentTimestamp - $lastPurgeTimestamp);
                $minutesRemaining = floor($timeRemaining / 60);
                $secondsRemaining = $timeRemaining % 60;
                $output .= $this->displayError(
                    $this->l(
                        sprintf(
                            'Cache purge request failed. Please wait %d minutes and %d seconds before trying again.',
                            $minutesRemaining,
                            $secondsRemaining
                        )
                    )
                );
                $this->addLog('TOTAL', 'KO: Cooldown', $adminInfo, null);
            } else {
                $result = $this->processPurgeCache();
                $status = $result['status'];
                if ($status == 200) {
                    $this->addLog('TOTAL', 'OK', $adminInfo, null);
                    Configuration::updateValue('ISFPLUGIN_PurgeCache', $currentTimestamp);
                    $output .= $this->displayConfirmation($this->l('Cache purge request successful.'));
                } else {
                    $this->addLog('TOTAL', 'KO: Purge failed', $adminInfo, null);
                    $output .= $this->displayError($this->l('Cache purge request failed.'));
                }
            }
        }
        $apiKey = Configuration::get('ISFPLUGIN_API_KEY');
        if (empty($apiKey)) {
            $output .= $this->displayError($this->l('No API Key has been set. Please verify your API Key.'));
            $this->addLog('KEY', 'KO: Empty key', $adminInfo, null);
        } else {
            if (Configuration::get('ISFPLUGIN_API_KEY_VERIFIED')) {
                $output .= $this->displayConfirmation($this->l('API Key verified successfully.'));
                $this->cleanOldLogs();
            } else {
                $this->addLog('KEY', 'KO: Verification failed', $adminInfo, $apiKey);
                $output .= $this->displayError($this->l('API Key not verified. Please verify your API Key.'));
            }
        }
        return $output . $this->renderForm();
    }
    private function verifyApiKey($apiKey)
    {
        $url = 'https://app.ismartframe.com/api/v1/cache/purge?api_key=' . urlencode($apiKey);
        $data = json_encode(
            [
                'url' => ['/testApiKey'],
            ]
        );
        $response = $this->makeApiRequest($url, $data);
        return $response;
    }
    public function renderForm()
    {
        $homeLabel = $this->l('Home');
        $logsLabel = $this->l('Logs');
        $documentationUrl = 'https://app.ismartframe.com/projects';
        $imagePath = $this->_path . 'views/img/logo ismartframe.png';
        $imageAlt = $this->l('Descrizione dell\'immagine');
        $firstTimeLabel = $this->l('First time on iSmartFrame?');
        $getStartedLabel = $this->l('Get started now,');
        $installLabel = $this->l('install the app and accelerate your website speed.');
        $howToGenerateApiKeyLabel = $this->l('How to Generate Your API Key');
        $stepsLabel = $this->l('To generate your API key, follow these steps:');
        $step1 = $this->l('Log in to your iSmartFrame account') . ' <a href="https://app.ismartframe.com/projects" target="_blank">' . $this->l('app.ismartframe.com') . '</a>';
        $step2 = $this->l('Navigate to your domain section in your dashboard.');
        $step3 = $this->l('Check the "API Key" section in your dashboard.');
        $step4 = $this->l('Click "Generate API Key" and copy it.');
        $step5 = $this->l('Paste it into the field in the plugin settings.');
        $apiKeyVerified = Configuration::get('ISFPLUGIN_API_KEY_VERIFIED');
        $fullCacheResetLabel = $this->l('Do you need a full cache reset?');
        $warningLabel = $this->l('Warning: This operation is not immediate and will slow down the entire site and decrease performance until the cache is correctly repopulated.');
        $clearCacheLabel = $this->l('Clear cache');
        $confirmationLabel = $this->l('Are you sure you want to purge the cache?');
        $yesLabel = $this->l('Yes');
        $noLabel = $this->l('No');
        $howIsfWorksLabel = $this->l('How iSmartFrame Purge Cache Plugin Works:');
        $autoCachePurgeLabel = $this->l('Automatic Cache Purge:');
        $autoCachePurgeDesc = $this->l('Automatically clears cache whenever new content is published or updates are made, ensuring that only the updated page is affected.');
        $targetedCacheManagementLabel = $this->l('Targeted Cache Management:');
        $targetedCacheManagementDesc = $this->l('Purges cache only for the specific page or tag, without impacting the entire website’s cache, keeping the rest of the site fast and efficient.');
        $customizableCacheControlLabel = $this->l('Customizable Cache Control:');
        $customizableCacheControlDesc = $this->l('Offers the ability to purge cache by specific tags or custom URLs, giving you more control and flexibility in managing what is cached.');
        $completeWebsiteCacheClearingLabel = $this->l('Complete Website Cache Clearing:');
        $completeWebsiteCacheClearingDesc = $this->l('Enables a full cache clearing for the entire website, pulling fresh data directly from the origin server to ensure up-to-date content everywhere.');
        $learnMoreLabel = $this->l('Learn more on how iSmartFrame works:');
        $howItWorksUrl = 'https://www.ismartframe.com/en/how-ismartframe-works';
        $howItWorksLink = $this->l('LINK a How it Works');
        $exploreLogsLabel = $this->l('Explore Logs');
        $lastLogsLabel = $this->l('Last 300 logs');
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('IsmartFrame plugin options'),
                ],
                'input' => [
                    [
                        'type' => 'text',
                        'label' => $this->l('API Key'),
                        'name' => 'ISFPLUGIN_API_KEY',
                        'size' => 40,
                        'required' => true,
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Domain'),
                        'name' => 'ISFPLUGIN_DOMAIN',
                        'size' => 40,
                        'readonly' => true,
                        'required' => true,
                        'value' => Tools::getShopDomain(),
                    ],
                ],
                'buttons' => [
                    'verify_api_key' => [
                        'title' => $this->l('Save and verify API Key'),
                        'name' => 'verifyApiKey',
                        'type' => 'submit',
                        'class' => 'btn btn-default pull-right btn-justify-start',
                    ],
                ],
            ],
        ];
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submit' . $this->name;
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFormValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];
        $form = $helper->generateForm([$fields_form]);

        $output = '';
        if (Tools::isSubmit('explore_logs')) {
            $output = $this->renderLogs(300);
        }
        $this->context->smarty->assign([
            'howToGenerateApiKeyLabel' => $howToGenerateApiKeyLabel,
            'stepsLabel' => $stepsLabel,
            'step1' => $step1,
            'step2' => $step2,
            'step3' => $step3,
            'step4' => $step4,
            'step5' => $step5,
            'homeLabel' => $homeLabel,
            'logsLabel' => $logsLabel,
            'documentationUrl' => $documentationUrl,
            'imagePath' => $imagePath,
            'imageAlt' => $imageAlt,
            'firstTimeLabel' => $firstTimeLabel,
            'getStartedLabel' => $getStartedLabel,
            'installLabel' => $installLabel,
            'form' => $form,
            'apiKeyVerified' => $apiKeyVerified,
            'fullCacheResetLabel' => $fullCacheResetLabel,
            'warningLabel' => $warningLabel,
            'clearCacheLabel' => $clearCacheLabel,
            'confirmationLabel' => $confirmationLabel,
            'yesLabel' => $yesLabel,
            'noLabel' => $noLabel,
            'howIsfWorksLabel' => $howIsfWorksLabel,
            'autoCachePurgeLabel' => $autoCachePurgeLabel,
            'autoCachePurgeDesc' => $autoCachePurgeDesc,
            'targetedCacheManagementLabel' => $targetedCacheManagementLabel,
            'targetedCacheManagementDesc' => $targetedCacheManagementDesc,
            'customizableCacheControlLabel' => $customizableCacheControlLabel,
            'customizableCacheControlDesc' => $customizableCacheControlDesc,
            'completeWebsiteCacheClearingLabel' => $completeWebsiteCacheClearingLabel,
            'completeWebsiteCacheClearingDesc' => $completeWebsiteCacheClearingDesc,
            'learnMoreLabel' => $learnMoreLabel,
            'howItWorksUrl' => $howItWorksUrl,
            'howItWorksLink' => $howItWorksLink,
            'exploreLogsLabel' => $exploreLogsLabel,
            'lastLogsLabel' => $lastLogsLabel,
            'logs' => $output,
        ]);
        return $this->display(__FILE__, 'views/templates/hook/bigform.tpl');
    }
    public function getLoggedAdminInfo()
    {
        $context = Context::getContext();
        if (isset($context) && isset($context->employee) && $context->employee->id) {
            $employeeId = $context->employee->id;
            $firstName = $context->employee->firstname;
            $lastName = $context->employee->lastname;
            $fullName = $firstName . ' ' . $lastName;
            return $employeeId . ' - ' . $fullName;
        } else {
            return null;
        }
    }
    private function renderLogs($numberOfLogs = 300)
    {
        $localTimezone = new DateTimeZone('Europe/Rome');
        $logs = $this->getLogs($numberOfLogs);
        if (empty($logs)) {
            return nl2br('No data available');
        }
        $output = [];
        foreach ($logs as $log) {
            $timestamp = isset($log['timestamp']) ? $log['timestamp'] : 'N/A';
            try {
                $dateTime = new DateTime($timestamp, new DateTimeZone('UTC'));
                $dateTime->setTimezone($localTimezone);
                $formattedTimestamp = $dateTime->format('d-m-Y H:i:s');
            } catch (Exception $e) {
                $formattedTimestamp = 'N/A';
            }
            $logObject = [
                'timestamp' => $formattedTimestamp,
                'operationType' => $log['operationType'] ?? 'N/A',
                'result' => $log['result'] ?? 'N/A',
                'userId' => $log['userId'] ?? 'N/A',
                'object' => $log['object'] ?? 'N/A',
            ];
            $output[] = json_encode($logObject, JSON_UNESCAPED_SLASHES);
        }
        return nl2br(implode('\n', $output));
    }
    public function processPurgeCache()
    {
        $apiKey = Configuration::get('ISFPLUGIN_API_KEY');
        $url = 'https://app.ismartframe.com/api/v1/cache/purge/pattern?api_key=' . urlencode($apiKey);
        $data = json_encode(
            [
                'pattern' => '(.*?)',
            ]
        );
        $response = $this->makeApiRequest($url, $data);
        return $response;
    }
    public function getConfigFormValues()
    {
        return [
            'ISFPLUGIN_API_KEY' => Configuration::get('ISFPLUGIN_API_KEY'),
            'ISFPLUGIN_DOMAIN' => Tools::getShopDomain(),
        ];
    }
    private function makeApiRequest($url, $data = null)
    {
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        $headers = [
            'Content-Type: application/json',
        ];
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        if ($data) {
            curl_setopt($curl, CURLOPT_POST, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }
        $response = curl_exec($curl);
        $status = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        $adminInfo = $this->getLoggedAdminInfo();
        if ($response === false) {
            $error = curl_error($curl);
            $this->addLog('API', 'KO: ' . $error, $adminInfo, $url);
        }
        curl_close($curl);
        return ['status' => $status, 'response' => $response];
    }
}
